#   (c) by zucKi    http://ircbot.bf-scene.de
#
#
#   DONT MODIFY THIS FILE
#   You are free to use this class for your modules
#   You are not allowed to distribute this file with your modules
###################################################################

import host
import bf2
import bf2.Timer
from bf2.stats.constants import *
from bf2.stats.stats import getStatsMap
import socket
import errno
from timeelapsed import *
from constants import *
from gamemaps import *

import sys
import traceback
import types

options = {
    "port": "4712",
    "password": "ircbot",
    "ip": "0.0.0.0"
}

def parseConfig():
    fn = "admin/bf_live/bf_live.con"
    try:
        config = open(fn, "r")
        lineNo = 0
        for line in config:
            lineNo += 1
            if line.strip() != "" and line.strip() != "\n":
                try:
                    (key, value) = line.split("=")
                    key = key.strip()
                    value = value.strip()

                    options[key] = value
                except ValueError:
                    print "warning: syntax error in '%s' on line %d" % (fn, lineNo)
    except IOError, detail:
        print "warning: couldn\'t read '%s': %s" % (fn, detail)
                
# A stateful output buffer that knows how to enqueue data and ship it out
# without blocking.
class OutputBuffer(object):

    def __init__(self, sock):
        self.sock = sock
        self.data = []
        self.index = 0

    def enqueue(self, str):
        self.data.append(str)

    def update(self):
        while len(self.data) > 0:
            try:
                item = self.data[0]
                scount = self.sock.send(item[self.index:])
                self.index += scount
                if self.index == len(item):
                    del self.data[0]
                    self.index = 0
            except socket.error, detail:
                if detail[0] != errno.EWOULDBLOCK:
                    return detail[1]
        return None


# Each TCP connection is represented by an object of this class.
class Bf2botConnection(object):

    def __init__(self, srv, sock, addr):
        print "new bf2`ircbot connection from %s:%d" % (addr[0], addr[1])
        self.server = srv
        self.sock = sock
        self.addr = addr
        self.sock.setblocking(0)
        self.buffer = ''
        self.outbuf = OutputBuffer(self.sock)
        self.authed = False
        #send welcome message to client
        self.write(IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_WELCOME + IRCBOT_COMMAND_SEPARATOR + IRCBOT_SEPARATOR + IRCBOT_SCRIPTVERSION)

    def write(self, text):
        self.outbuf.enqueue(text + IRCBOT_PACKET_SEPARATOR)

    def shutdown(self):
        if self.sock:
            self.sock.close()
        
    def update(self):
        err = None
        try:
            while not err:
                data = self.sock.recv(1024)
                if data:
                    self.buffer += data
                    while not err:
                        nlpos = self.buffer.find("\n")
                        if nlpos != -1:
                            self.server.parseCommand(self, self.buffer[0:nlpos])
                            self.buffer = self.buffer[nlpos+1:] # keep rest of buffer
                        else:
                            if len(self.buffer) > 128:
                                err = "data format error: no newline in message"
                            break
                else:
                    err = "peer disconnected"

        except socket.error, detail:
            if detail[0] != errno.EWOULDBLOCK:
                err = detail[1]

        if not err:
            err = self.outbuf.update()

        if err:
            print "bf2`ircbot: closing %s:%d: %s" % (self.addr[0], self.addr[1], err)
            try:
                self.sock.shutdown(2)
                self.sock.close()
            except:
                pass
            return 0
        else:
            return 1

class bf_liveServer(object):

    def __init__(self):
        self.port = int(options["port"])
        self.listen = 1
        self.peers = []
        self.openSocket()
        self.timer = bf2.Timer(self.onAdTimer, 600, 1)
        self.timer.setRecurring(600)
        self.roundTime = TimeElapsed()
        self.gamestatus = 0
        self.players = {}

        #handler !
        host.registerHandler("PlayerConnect", self.onPlayerConnect, 1)
        host.registerHandler("PlayerDisconnect", self.onPlayerDisconnect, 1)
        #host.registerHandler("PlayerChangeTeams", self.onPlayerChangeTeams, 1)
        host.registerHandler("PlayerKilled", self.onPlayerKilled, 1)
        host.registerHandler("PlayerDeath", self.onPlayerDeath, 1)
        host.registerHandler("PlayerSpawn", self.onPlayerSpawn, 1 )
        host.registerHandler("ChangedCommander", self.onChangedCommander, 1)
        host.registerHandler("ChatMessage", self.onChatMessage, 1)
        host.registerHandler("ControlPointChangedOwner", self.onCPStatusChange, 1)
        host.registerHandler("PlayerRevived", self.onPlayerRevived, 1)
        host.registerHandler("PlayerBanned", self.onPlayerBanned, 1)
        host.registerHandler("PlayerKicked", self.onPlayerKicked, 1)
        host.registerGameStatusHandler(self.onGameStatusChanged, 1)

        # bf2`ircbot commands
        self.bot_cmds = {
            IRCBOT_COMMAND_LOGIN: self.botcmd_login,
            IRCBOT_COMMAND_TICKETSTATUS: self.botcmd_ticketstatus,
            IRCBOT_COMMAND_TIMEELAPSED: self.botcmd_timeelapsed,
            IRCBOT_COMMAND_SERVERINFO: self.botcmd_serverinfo,
            IRCBOT_COMMAND_PLAYERLIST: self.botcmd_playerlist,
            IRCBOT_COMMAND_CPSTATUS: self.botcmd_controlpointstatus,
            IRCBOT_COMMAND_TOP3: self.botcmd_top3,
            IRCBOT_COMMAND_WINNER: self.botcmd_winner,
            IRCBOT_COMMAND_ALIVEPLAYERLIST: self.botcmd_aliveplayerlist
        }

    def sendToClients(self, text):
        for client in self.peers:
            if client.authed:
                client.write(text)

    # Called when the bfircbot send a request e.g. wanna see the playerlist
    def parseCommand(self, bot_client, cmd):
        cmd = cmd.strip()
        spacepos = cmd.find(' ')
        if spacepos == -1: spacepos=len(cmd)
        subcmd = cmd[0:spacepos]

        if subcmd != IRCBOT_COMMAND_LOGIN and not bot_client.authed:
            bot_client.write(IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_ERROR + IRCBOT_COMMAND_SEPARATOR + IRCBOT_SEPARATOR + str(IRCBOT_ERROR_NOTAUTHED))
        else:
            try:
                self.bot_cmds[subcmd](bot_client, cmd[spacepos+1:])
            except Exception:
                bot_client.write(IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_ERROR + IRCBOT_COMMAND_SEPARATOR + IRCBOT_SEPARATOR + str(IRCBOt_ERROR_UNKOWNCOMMAND) + IRCBOT_SEPARATOR + subcmd)

    def onAdTimer(self, data):
        host.rcon_invoke("game.sayAll \"*** This Server is using the bf_live module. |C1001http://ircbot.bf-scene.de|C1001 <> |C1001#bf.live|C1001 (quakenet) ***\"")

    def onPlayerConnect(self, p):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_CONNECT + IRCBOT_COMMAND_SEPARATOR
        ret += IRCBOT_SEPARATOR + str(p.index)
        ret += IRCBOT_SEPARATOR + p.getName()
        ret += IRCBOT_SEPARATOR + str(p.getTeam())

        self.players[p.index] = (p.getTeam(), p.getName())
        
        self.sendToClients(ret)
        
    def onPlayerDisconnect(self, p):
        if type(p) == types.IntType:
            #onPlayerDisconnect was fired manually (disconnect between mapchange)
            index = p
        else:
            #onPlayerDisconnect was fired by bf2
            index = p.index

        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_DISCONNECT + IRCBOT_COMMAND_SEPARATOR
        ret += IRCBOT_SEPARATOR + str(index)

        del self.players[index]
        
        self.sendToClients(ret)
        
    def onPlayerDeath(self, p, vehicle):
        if self.players[p.index][0] != p.getTeam():
            self.onPlayerChangeTeams(p, 1)

    def onPlayerSpawn(self, p, s):
        if self.players[p.index][0] != p.getTeam():
            self.onPlayerChangeTeams(p, 1)
        

    # the kill packet has three versions:
    #   1. kill, teamkill and run_over are the same:
    #       attacker_id | attacker_kit_id | attacker_vehicle_id | attacker_score | attacker_kills | attacker_deaths |
    #       && victim_id | victim_kit_id | victim_vehicle_id | victim_score | victim_kills | victim_deaths | weapon_id
    #   2. suicide:
    #       victim_id | victim_kit_id | victim_score | victim_kills | victim_deaths
    #   3. killed himself:
    #       victim_id | victim_kit_id | victim_vehicle_id | victim_score | victim_kills | victim_deaths | weapon_id
    def onPlayerKilled(self, victim, attacker, weapon, assists, object):
        try:
            ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_KILL + IRCBOT_COMMAND_SEPARATOR
            attacker_id = -1
            attacker_kit_id = -1
            attacker_vehicle_id = -1
            
            victim_id = victim.index
            victim_kit_id = -1

            kit = victim.getKit()
            if kit != None:
                victim_kit_id = getKitIndex(kit.templateName)

            victim_vehicle = victim.getVehicle()
            if victim_vehicle != None:
                victim_vehicle_id = getVehicleIndex(victim_vehicle.templateName)
            
            weapon_id = -1
            action_id = -1

            # check if killed by an empty vehicle in motion
            killedByEmptyVehicle = False
            if attacker == None and weapon == None and object != None:
                if hasattr(object, "lastDrivingPlayerIndex"):
                    # getting last driver of the vehicle -> he gets the kill
                    attacker = bf2.playerManager.getPlayerByIndex(object.lastDrivingPlayerIndex)
                    killedByEmptyVehicle = True

            # killed by a player
            if attacker != None:
                attacker_id = attacker.index
                # kill
                if attacker.getTeam() != victim.getTeam():
                    action_id = ACTION_KILL
                else:
                    # suicide (pressed the button)
                    if attacker.index == victim.index:
                        action_id = ACTION_SUICIDE
                    # teamkill
                    else:
                        action_id = ACTION_TEAMKILL
            # suicide (fall down or something)
            else:
                action_id = ACTION_SUICIDE

            # handling vehicle
            if killedByEmptyVehicle:
                attacker_vehicle_id = getVehicleIndex(object.templateName)
            elif attacker != None:
                attacker_vehicle_id = getVehicleIndex(attacker.getVehicle().templateName)

            # handling weapon
            if weapon != None:
                weapon_id = getWeaponIndex(weapon.templateName)
                if action_id == ACTION_SUICIDE:
                    action_id = ACTION_KILLED_HIMSELF
            elif attacker_vehicle_id != -1:
                if action_id == ACTION_KILL:
                    action_id = ACTION_ROADKILL
                else:
                    action_id = ACTION_ROADTEAMKILL                    

            # handling attacker kit
            if attacker != None:
                kit = attacker.getKit()
                if kit != None:
                    attacker_kit_id = getKitIndex(kit.templateName)

            #output
            ret += IRCBOT_SEPARATOR + str(action_id)
            if action_id == ACTION_KILL or action_id == ACTION_TEAMKILL or action_id == ACTION_ROADKILL or action_id == ACTION_ROADTEAMKILL:
                ret += IRCBOT_SEPARATOR + str(attacker_id)
                ret += IRCBOT_SEPARATOR + str(attacker_kit_id)
                ret += IRCBOT_SEPARATOR + str(attacker_vehicle_id)
                ret += IRCBOT_SEPARATOR + str(attacker.score.score)
                ret += IRCBOT_SEPARATOR + str(attacker.score.kills)
                ret += IRCBOT_SEPARATOR + str(attacker.score.deaths)
                ret += IRCBOT_SEPARATOR + str(victim_id)
                ret += IRCBOT_SEPARATOR + str(victim_kit_id)
                ret += IRCBOT_SEPARATOR + str(victim_vehicle_id)
                ret += IRCBOT_SEPARATOR + str(victim.score.score)
                ret += IRCBOT_SEPARATOR + str(victim.score.kills)
                ret += IRCBOT_SEPARATOR + str(victim.score.deaths)
                ret += IRCBOT_SEPARATOR + str(weapon_id)
            elif action_id == ACTION_SUICIDE:
                ret += IRCBOT_SEPARATOR + str(victim_id)
                ret += IRCBOT_SEPARATOR + str(victim_kit_id)
                ret += IRCBOT_SEPARATOR + str(victim.score.score)
                ret += IRCBOT_SEPARATOR + str(victim.score.kills)
                ret += IRCBOT_SEPARATOR + str(victim.score.deaths)
            elif action_id == ACTION_KILLED_HIMSELF:
                ret += IRCBOT_SEPARATOR + str(victim_id)
                ret += IRCBOT_SEPARATOR + str(victim_kit_id)
                ret += IRCBOT_SEPARATOR + str(victim_vehicle_id)
                ret += IRCBOT_SEPARATOR + str(victim.score.score)
                ret += IRCBOT_SEPARATOR + str(victim.score.kills)
                ret += IRCBOT_SEPARATOR + str(victim.score.deaths)
                ret += IRCBOT_SEPARATOR + str(weapon_id)

            self.sendToClients(ret)
        except Exception, e:
            pass


    #sends chatmessages to the bf2`ircbot
    def onChatMessage(self, player_id, message, channel, flags):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_CHAT + IRCBOT_COMMAND_SEPARATOR
        player_status = CHAT_PLAYER_ALIVE
        message = message.replace("|C1001", "")
        
        # find and delete HUD_... strings
        if channel == "ServerMessage":
            channel_id = CHAT_CHANNEL_SERVERMESSAGE
        elif channel == "Global":
            channel_id = CHAT_CHANNEL_GLOBAL
            # find and delete *DEAD* string (*\xa71DEAD\xa70*)
            # note: the string inside of \xa7 depends on the language!
            pos = message.find("*\xa7")
            if pos != -1:
                pos2 = message.find("*", pos+2)
                if pos2 != -1:
                    message = message[pos2+1:]
                    player_status = CHAT_PLAYER_DEAD
        else:
            return
       
        ret += IRCBOT_SEPARATOR + str(player_id) + IRCBOT_SEPARATOR + str(player_status) + IRCBOT_SEPARATOR + str(channel_id) + IRCBOT_SEPARATOR + message
        self.sendToClients(ret)

    #this will only be called manually.
    def onPlayerChangeTeams(self, p, humanHasSpawned):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_TEAMCHANGE + IRCBOT_COMMAND_SEPARATOR
        ret += IRCBOT_SEPARATOR + str(p.index) + IRCBOT_SEPARATOR + str(p.getTeam())
        
        self.players[p.index] = (p.getTeam(), p.getName())

        self.sendToClients(ret)


    def onGameStatusChanged(self, status):
        try:
            self.gamestatus = status
            
            ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_GAMESTATUS + IRCBOT_COMMAND_SEPARATOR
            ret += IRCBOT_SEPARATOR + str(status)

            self.sendToClients(ret)

            #handling disconnects between mapchanges
            if status == bf2.GameStatus.Playing:
                #checking disconnects
                for index in self.players.keys():
                    valid_player = host.pmgr_isIndexValid(index)
                    #check if player is valid
                    if not valid_player:
                        if self.players.has_key(index):
                            #found a not valid index -> manually disconnect him
                            self.onPlayerDisconnect(index)

                #checking connects
                for p in bf2.playerManager.getPlayers():
                    if not self.players.has_key(p.index) or p.getName() != self.players[p.index][1]:
                        #found a new player: manually connect him
                        self.onPlayerConnect(p)
        except Exception, e:
            pass

    def onCPStatusChange(self, cp, pos):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_CONTROLPOINT + IRCBOT_COMMAND_SEPARATOR
        scoringTeam = 0
        capture_status = 0
        # determine capture / neutralize / defend
        if cp.cp_getParam("team") != 0:
            if cp.cp_getParam("team") == 1:
                scoringTeam = 2
            else:
                scoringTeam = 1
            if pos:
                # defended flag
                capture_status = CAPTUREPOINT_DEFENDED
            else:
                # neutralized flag
                capture_status = CAPTUREPOINT_NEUTRALIZED
        else:
            if pos:
                # captured flag
                scoringTeam = cp.cp_getParam("flag")
                capture_status = CAPTUREPOINT_CAPTURED
            else:
                # hit bottom, but still neutral
                # no message!!!
                return
        ret += IRCBOT_SEPARATOR + cp.getTemplateProperty("controlPointId") + IRCBOT_SEPARATOR + str(capture_status) + IRCBOT_SEPARATOR + str(scoringTeam)
        self.sendToClients(ret)

    def onPlayerRevived(self, dead, healer):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_REVIVE + IRCBOT_COMMAND_SEPARATOR
        ret += IRCBOT_SEPARATOR + str(healer.index) + IRCBOT_SEPARATOR + str(dead.index)
        self.sendToClients(ret)

    def onChangedCommander(self, team, oldCmd, newCmd):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_COMMANDER + IRCBOT_COMMAND_SEPARATOR
        oc = -1
        if oldCmd: oc = oldCmd.index
        nc = -1
        if newCmd: nc = newCmd.index

        action_id = -1
        if nc == -1:
            action_id = COMMANDER_LEFT
            ret += IRCBOT_SEPARATOR + str(action_id) + IRCBOT_SEPARATOR + str(oc)
        elif oc == -1:
            action_id = COMMANDER_JOIN
            ret += IRCBOT_SEPARATOR + str(action_id) + IRCBOT_SEPARATOR + str(nc)

        self.sendToClients(ret)

    def onPlayerBanned(self, p, time, type):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_BANNED + IRCBOT_COMMAND_SEPARATOR

        ret += IRCBOT_SEPARATOR + str(p.index)

        self.sendToClients(ret)

    def onPlayerKicked(self, p):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_KICKED + IRCBOT_COMMAND_SEPARATOR

        ret += IRCBOT_SEPARATOR + str(p.index)
        
        self.sendToClients(ret)

    # Sets up the listening TCP bf_live socket. 
    def openSocket(self):
        try:
            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.sock.bind((options["ip"], self.port))
            self.sock.listen(self.listen)
            self.sock.setblocking(0)
        except socket.error, detail:
            print "failed to bind bf_live socket"

    def update(self):
        if not self.sock: return

        try:
            conn, peeraddr = self.sock.accept()
            self.peers.append(Bf2botConnection(self, conn, peeraddr))
        except socket.error, detail:
            if detail[0] != errno.EWOULDBLOCK:
                raise socket.error, detail

        disc = []
        for client in self.peers:
            if not client.update(): disc.append(client)

        self.peers = filter(lambda x: x not in disc, self.peers)

    def shutdown(self):
        if self.sock:
            for client in self.peers:
                client.shutdown()
            self.sock.close()

    # Auth for the bfircbot
    def botcmd_login(self, bot_client, cmd):
        success = 0
        if cmd.strip() == options["password"]:
            print "bf_live: bf2`ircbot connection from %s:%d logged on" % bot_client.addr
            success = 1
                        
        if success:
            bot_client.authed = True
            bot_client.write(IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_AUTH + IRCBOT_COMMAND_SEPARATOR + IRCBOT_SEPARATOR + str(IRCBOT_AUTH_SUCCESFUL))
        else:
            bot_client.write(IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_AUTH + IRCBOT_COMMAND_SEPARATOR + IRCBOT_SEPARATOR + str(IRCBOT_AUTH_FAILED))
                    
    def botcmd_ticketstatus(self, bot_client, cmd):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_TICKETSTATUS + IRCBOT_COMMAND_SEPARATOR

        ret += IRCBOT_SEPARATOR + str(host.sgl_getParam("tickets", 1, 1))       # team 1 tickets
        ret += IRCBOT_SEPARATOR + str(host.sgl_getParam("ticketChangePerSecond", 1, 1)) # team 1 ticket loss per sec
        ret += IRCBOT_SEPARATOR + str(host.sgl_getParam("tickets", 2, 1))       # team 2 tickets
        ret += IRCBOT_SEPARATOR + str(host.sgl_getParam("ticketChangePerSecond", 2, 1)) # team 1 ticket loss per sec

        bot_client.write(ret)

    def botcmd_timeelapsed(self, bot_client, cmd):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_TIMEELAPSED + IRCBOT_COMMAND_SEPARATOR
        
        ret += IRCBOT_SEPARATOR + str(self.roundTime.getTime())
        
        bot_client.write(ret)

    def botcmd_serverinfo(self, bot_client, cmd):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_SERVERINFO + IRCBOT_COMMAND_SEPARATOR

        name = host.rcon_invoke("sv.serverName")
        len_name = len(name)
        if len_name > 1:
            name = name[:len_name-1]
        ret += IRCBOT_SEPARATOR + name

        port = host.rcon_invoke("sv.serverPort")
        len_port = len(port)
        if len_port > 1:
            port = port[:len_port-1]
        ret += IRCBOT_SEPARATOR + port
        ret += IRCBOT_SEPARATOR + str(bf2.serverSettings.getMaxPlayers())
        ret += IRCBOT_SEPARATOR + str(host.pmgr_getNumberOfPlayers())
        ret += IRCBOT_SEPARATOR + str(getMapIndex(bf2.serverSettings.getMapName()))
        ret += IRCBOT_SEPARATOR + str(host.sgl_getParam("teamName", 1, 0)) # TEAM 1
        ret += IRCBOT_SEPARATOR + str(host.sgl_getParam("teamName", 2, 0)) # TEAM 2
        ret += IRCBOT_SEPARATOR + bf2.serverSettings.getGameMode()

        modDirs = str(host.sgl_getModDirectory()).replace("\\", "/").split("/")
        modDir = modDirs[modDirs.__len__() - 1]
        ret += IRCBOT_SEPARATOR + str(modDir) # mod running

        ret += IRCBOT_SEPARATOR + str(host.sgl_getWorldSize())
        ret += IRCBOT_SEPARATOR + str(bf2.serverSettings.getTimeLimit())
        ret += IRCBOT_SEPARATOR + str(self.roundTime.getTime()) #current roundtime (only importent when the client connects in the middle of a game)
        ret += IRCBOT_SEPARATOR + str(self.gamestatus)
        bot_client.write(ret)

    def botcmd_playerlist(self, bot_client, cmd):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_PLAYERLIST + IRCBOT_COMMAND_SEPARATOR

        for p in bf2.playerManager.getPlayers():
            ret += IRCBOT_SEPARATOR + str(p.index)
            ret += IRCBOT_SEPARATOR + p.getName()
            ret += IRCBOT_SEPARATOR + str(p.getTeam())
            ret += IRCBOT_SEPARATOR + str(p.isCommander())
            ret += IRCBOT_SEPARATOR + str(p.score.score)
            ret += IRCBOT_SEPARATOR + str(p.score.kills)
            ret += IRCBOT_SEPARATOR + str(p.score.deaths)
        bot_client.write(ret)

    def botcmd_controlpointstatus(self, bot_client, cmd):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_CPSTATUS + IRCBOT_COMMAND_SEPARATOR

        controlpoints = bf2.objectManager.getObjectsOfType("dice.hfe.world.ObjectTemplate.ControlPoint")
        for cp in controlpoints:
            cp_id = cp.getTemplateProperty("controlPointId")
            team_id = cp.cp_getParam("team")
            ret += IRCBOT_SEPARATOR + str(cp_id) + IRCBOT_SEPARATOR + str(team_id)

        bot_client.write(ret)

    def botcmd_aliveplayerlist(self, bot_client, cmd):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_ALIVEPLAYERLIST + IRCBOT_COMMAND_SEPARATOR
        ret += IRCBOT_SEPARATOR + str(cmd)
        
        for p in bf2.playerManager.getPlayers():
            if p.isAlive() and not p.isManDown() and p.getTeam() == int(cmd):
                ret += IRCBOT_SEPARATOR + str(p.index)
        
        bot_client.write(ret)

    def botcmd_winner(self, bot_client, cmd):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_WINNER + IRCBOT_COMMAND_SEPARATOR

        ret += IRCBOT_SEPARATOR + str(bf2.gameLogic.getWinner())
        ret += IRCBOT_SEPARATOR + str(bf2.gameLogic.getVictoryType())

        bot_client.write(ret)

    def botcmd_top3(self, bot_client, cmd):
        self.sendTop3(bot_client)

    def sendTop3(self, bot_client):
        ret = IRCBOT_COMMAND_SEPARATOR + IRCBOT_COMMAND_TOP3 + IRCBOT_COMMAND_SEPARATOR

        playerlist = bf2.playerManager.getPlayers()
        for i in range(3):
            top = None
            for p in playerlist:
                if top == None:
                    top = p
                top = self.getTopPlayer(top, p)
            if top:
                ret += IRCBOT_SEPARATOR + str(top.index)
                ret += IRCBOT_SEPARATOR + str(top.score.score)
                ret += IRCBOT_SEPARATOR + str(top.score.kills)
                ret += IRCBOT_SEPARATOR + str(top.score.deaths)
                playerlist.remove(top)
        
        bot_client.write(ret)

    def getTopPlayer(self, p1, p2):
        if p1 == None:
            return p2
        if p2 == None:
            return p1

        if p1.score.score > p2.score.score:
            return p1
        elif p1.score.score == p2.score.score:
            if p1.score.kills > p2.score.kills:
                return p1
            elif p1.score.kills == p2.score.kills:
                if p1.score.deaths > p2.score.deaths:
                    return p2
                else:
                    return p1
            else:
                return p2
        else:
            return p2


# loading config file
parseConfig()
